/*
 * (C) Copyright 2014 Kurento (http://kurento.org/)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser General Public License
 * (LGPL) version 2.1 which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl-2.1.html
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */

#include "OptimizedGstBuffer.h"

#define RESET_OPTIMIZED_GST_BUFFER(buffer_ptr) ({        \
  GstBuffer_t *tbuffer = (GstBuffer_t *) (buffer_ptr);   \
  tbuffer->data.buf = NULL;                              \
  tbuffer->data.size = 0;                                \
})

#define MAP_OPTIMIZED_GST_BUFFER(buffer_ptr, map_ptr) ({ \
  GstBuffer_t *tbuffer = (GstBuffer_t *) (buffer_ptr);   \
  tbuffer->data.buf = (map_ptr)->data;                   \
  tbuffer->data.size = (map_ptr)->size;                 \
})

#define ENCODE_OPTIMIZED_GST_BUFFER(struct_ptr, enc_f, args...) ({             \
  OptimizedGstBuffer_t *_obuffer = (OptimizedGstBuffer_t *) (struct_ptr);      \
  asn_enc_rval_t _ret;                                                         \
  GstMapInfo _map;                                                             \
                                                                               \
  if (_obuffer->gstbuffer == NULL) {                                           \
    _ret = enc_f (args);                                                       \
  } else {                                                                     \
    gst_buffer_map ((GstBuffer *) _obuffer->gstbuffer, &_map, GST_MAP_READ);   \
                                                                               \
    if (_map.size > 0) {                                                       \
      MAP_OPTIMIZED_GST_BUFFER (_obuffer, &_map);                              \
    }                                                                          \
                                                                               \
    _ret = enc_f (args);                                                       \
                                                                               \
    RESET_OPTIMIZED_GST_BUFFER (_obuffer);                                     \
                                                                               \
    gst_buffer_unmap ((GstBuffer *) _obuffer->gstbuffer, &_map);               \
  }                                                                            \
  _ret;                                                                        \
})

static asn_TYPE_member_t asn_MBR_OptimizedGstBuffer_1[] = {
  {ATF_NOFLAGS, 0, offsetof (struct GstBuffer, pts),
        (ASN_TAG_CLASS_UNIVERSAL | (22 << 2)),
        0,
        &asn_DEF_GstClockTime,
        0,                      /* Defer constraints checking to the member type */
        0,                      /* No PER visible constraints */
        0,
      "pts"},
  {ATF_NOFLAGS, 0, offsetof (struct GstBuffer, dts),
        (ASN_TAG_CLASS_UNIVERSAL | (22 << 2)),
        0,
        &asn_DEF_GstClockTime,
        0,                      /* Defer constraints checking to the member type */
        0,                      /* No PER visible constraints */
        0,
      "dts"},
  {ATF_NOFLAGS, 0, offsetof (struct GstBuffer, duration),
        (ASN_TAG_CLASS_UNIVERSAL | (22 << 2)),
        0,
        &asn_DEF_GstClockTime,
        0,                      /* Defer constraints checking to the member type */
        0,                      /* No PER visible constraints */
        0,
      "duration"},
  {ATF_NOFLAGS, 0, offsetof (struct GstBuffer, offset),
        (ASN_TAG_CLASS_UNIVERSAL | (22 << 2)),
        0,
        &asn_DEF_GTypeUInt64,
        0,                      /* Defer constraints checking to the member type */
        0,                      /* No PER visible constraints */
        0,
      "offset"},
  {ATF_NOFLAGS, 0, offsetof (struct GstBuffer, offset_end),
        (ASN_TAG_CLASS_UNIVERSAL | (22 << 2)),
        0,
        &asn_DEF_GTypeUInt64,
        0,                      /* Defer constraints checking to the member type */
        0,                      /* No PER visible constraints */
        0,
      "offset-end"},
  {ATF_NOFLAGS, 0, offsetof (struct GstBuffer, flags),
        (ASN_TAG_CLASS_UNIVERSAL | (3 << 2)),
        0,
        &asn_DEF_GstBufferFlags,
        0,                      /* Defer constraints checking to the member type */
        0,                      /* No PER visible constraints */
        0,
      "flags"},
  {ATF_NOFLAGS, 0, offsetof (struct GstBuffer, data),
        (ASN_TAG_CLASS_UNIVERSAL | (4 << 2)),
        0,
        &asn_DEF_OCTET_STRING,
        0,                      /* Defer constraints checking to the member type */
        0,                      /* No PER visible constraints */
        0,
      "data"},
};
static ber_tlv_tag_t asn_DEF_OptimizedGstBuffer_tags_1[] = {
  (ASN_TAG_CLASS_UNIVERSAL | (16 << 2))
};

static asn_TYPE_tag2member_t asn_MAP_OptimizedGstBuffer_tag2el_1[] = {
  {(ASN_TAG_CLASS_UNIVERSAL | (3 << 2)), 5, 0, 0},      /* flags at 271 */
  {(ASN_TAG_CLASS_UNIVERSAL | (4 << 2)), 6, 0, 0},      /* data at 272 */
  {(ASN_TAG_CLASS_UNIVERSAL | (22 << 2)), 0, 0, 4},     /* pts at 266 */
  {(ASN_TAG_CLASS_UNIVERSAL | (22 << 2)), 1, -1, 3},    /* dts at 267 */
  {(ASN_TAG_CLASS_UNIVERSAL | (22 << 2)), 2, -2, 2},    /* duration at 268 */
  {(ASN_TAG_CLASS_UNIVERSAL | (22 << 2)), 3, -3, 1},    /* offset at 269 */
  {(ASN_TAG_CLASS_UNIVERSAL | (22 << 2)), 4, -4, 0}     /* offset-end at 270 */
};

static asn_SEQUENCE_specifics_t asn_SPC_OptimizedGstBuffer_specs_1 = {
  sizeof (struct GstBuffer),
  offsetof (struct GstBuffer, _asn_ctx),
  asn_MAP_OptimizedGstBuffer_tag2el_1,
  7,                            /* Count of tags in the map */
  0, 0, 0,                      /* Optional elements (not needed) */
  -1,                           /* Start extensions */
  -1                            /* Stop extensions */
};

/* Custom encode functions which overwrite the GstBuffer_t ones so that */
/* we can map and unmap the buffer without copying it when encoding     */

static asn_enc_rval_t
OptimizedGstBuffer_der_encoder_f (struct asn_TYPE_descriptor_s *type_descriptor,
    void *struct_ptr, int tag_mode, ber_tlv_tag_t tag,
    asn_app_consume_bytes_f * consume_bytes_cb, void *app_key)
{
  return ENCODE_OPTIMIZED_GST_BUFFER (struct_ptr, SEQUENCE_encode_der,
      type_descriptor, struct_ptr, tag_mode, tag, consume_bytes_cb, app_key);
}

static asn_enc_rval_t
OptimizedGstBuffer_xer_encoder_f (struct asn_TYPE_descriptor_s *type_descriptor,
    void *struct_ptr, int ilevel, enum xer_encoder_flags_e xer_flags,
    asn_app_consume_bytes_f * consume_bytes_cb, void *app_key)
{
  return ENCODE_OPTIMIZED_GST_BUFFER (struct_ptr, SEQUENCE_encode_xer,
      type_descriptor, struct_ptr, ilevel, xer_flags, consume_bytes_cb,
      app_key);
}

static asn_enc_rval_t
OptimizedGstBuffer_per_encoder_f (struct asn_TYPE_descriptor_s *type_descriptor,
    asn_per_constraints_t * constraints, void *struct_ptr,
    asn_per_outp_t * per_output)
{
  return ENCODE_OPTIMIZED_GST_BUFFER (struct_ptr, SEQUENCE_encode_uper,
      type_descriptor, constraints, struct_ptr, per_output);
}

/* Free function will first release the buffer and then the rest of */
/* structures contained in GstBuffer_t excepting the OCTET_STRING   */
void
OptimizedGstBuffer_free (struct asn_TYPE_descriptor_s *type_descriptor,
    void *struct_ptr, int free_contents_only)
{
  OptimizedGstBuffer_t *obuffer = (OptimizedGstBuffer_t *) struct_ptr;

  if (obuffer->gstbuffer != NULL) {
    gst_buffer_unref (obuffer->gstbuffer);

    /* Buffer memory is owned by GstBuffer. Do not release OCTET_STRING */
    /* set pointer to NULL instead. */
    RESET_OPTIMIZED_GST_BUFFER (obuffer);
  }

  SEQUENCE_free (type_descriptor, struct_ptr, free_contents_only);
}

asn_TYPE_descriptor_t asn_DEF_OptimizedGstBuffer = {
  "OptimizedGstBuffer",
  "OptimizedGstBuffer",
  OptimizedGstBuffer_free,
  SEQUENCE_print,
  SEQUENCE_constraint,
  SEQUENCE_decode_ber,
  OptimizedGstBuffer_der_encoder_f,
  SEQUENCE_decode_xer,
  OptimizedGstBuffer_xer_encoder_f,
  SEQUENCE_decode_uper,
  OptimizedGstBuffer_per_encoder_f,
  0,                            /* Use generic outmost tag fetcher */
  asn_DEF_OptimizedGstBuffer_tags_1,
  sizeof (asn_DEF_OptimizedGstBuffer_tags_1)
      / sizeof (asn_DEF_OptimizedGstBuffer_tags_1[0]),  /* 1 */
  asn_DEF_OptimizedGstBuffer_tags_1,    /* Same as above */
  sizeof (asn_DEF_OptimizedGstBuffer_tags_1)
      / sizeof (asn_DEF_OptimizedGstBuffer_tags_1[0]),  /* 1 */
  0,                            /* No PER visible constraints */
  asn_MBR_OptimizedGstBuffer_1,
  7,                            /* Elements count */
  &asn_SPC_OptimizedGstBuffer_specs_1   /* Additional specs */
};
