/*
 * (C) Copyright 2014 Kurento (http://kurento.org/)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser General Public License
 * (LGPL) version 2.1 which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl-2.1.html
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */

#ifndef _KURENTO_MARSHAL_TEST_
#define _KURENTO_MARSHAL_TEST_

#define CREATE_SINGLE_GVALUE(gtype, set_func, val) ({  \
  GValue *__gvalue;                                    \
  __gvalue = g_slice_new0 (GValue);                    \
  g_value_init (__gvalue, gtype);                      \
  set_func (__gvalue, (val));                          \
  __gvalue;                                            \
})

#define CREATE_RANGE_GVALUE(gtype, set_func, val1, val2) ({   \
  GValue *__gvalue;                                           \
  __gvalue = g_slice_new0 (GValue);                           \
  g_value_init (__gvalue, gtype);                             \
  set_func (__gvalue, (val1), (val2));                        \
  __gvalue;                                                   \
})

#define TEST_BASE_CASE(rules, CREATE_GVALUE_MACRO, size, testn) ({            \
  GValue *val1 = NULL, *val2 = NULL;                                          \
  GError *err = NULL;                                                         \
  gchar buf[size];                                                            \
  gsize s;                                                                    \
                                                                              \
  val1 = CREATE_GVALUE_MACRO;                                                 \
                                                                              \
  /* Encode GVal 1 */                                                         \
  if ((s = enc_GValue (rules, val1, buf, size, &err)) < 0) {                  \
    GST_ERROR ("%s", err->message);                                           \
    fail ("Failed encoding GValue %s", g_type_name (G_VALUE_TYPE (val1)));    \
    goto end_##testn;                                                         \
  }                                                                           \
                                                                              \
  /* Encode GVal 2 */                                                         \
  if (!dec_GValue (rules, buf, s, &val2, &err)) {                             \
    GST_ERROR ("%s", err->message);                                           \
    goto end_##testn;                                                         \
  }                                                                           \
                                                                              \
  if (!gvalue_are_equal (val1, val2)) {                                       \
    GST_ERROR ("Value encoded and value decoded are not the same");           \
    fail ("Marshal did not succeed");                                         \
  }                                                                           \
                                                                              \
end_##testn:                                                                  \
  if (err != NULL)                                                            \
    g_error_free (err);                                                       \
                                                                              \
  if (val1 != NULL) {                                                         \
    g_value_unset (val1);                                                     \
    g_slice_free (GValue, val1);                                              \
  }                                                                           \
                                                                              \
  if (val2 != NULL) {                                                         \
    g_value_unset (val2);                                                     \
    g_slice_free (GValue, val2);                                              \
  }                                                                           \
})

gboolean gvalue_are_equal (const GValue * v1, const GValue * v2);

#endif /* _KURENTO_MARSHAL_TEST_ */
