/*
 * (C) Copyright 2014 Kurento (http://kurento.org/)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser General Public License
 * (LGPL) version 2.1 which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl-2.1.html
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */

#include <gst/check/gstcheck.h>
#include <gst/gst.h>
#include "marshal_types.h"
#include "marshal_test.h"

#define BUFFER_LENGTH (1024 * 6)        /* bytes */
#define TEST_BUFFER "This is just for testing purposes"

static void
test_int_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_INT, g_value_set_int,
          G_MAXINT), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_INT, g_value_set_int,
          (G_MAXINT / 2)), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_INT, g_value_set_int, 0),
      BUFFER_LENGTH, 2);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_INT, g_value_set_int,
          (G_MAXINT / 2)), BUFFER_LENGTH, 3);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_INT, g_value_set_int,
          G_MININT), BUFFER_LENGTH, 4);
}

GST_START_TEST (marshal_int)
{
  test_int_GValue (KURENTO_MARSHALL_BER);
  test_int_GValue (KURENTO_MARSHALL_XER);
  test_int_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_uint_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_UINT, g_value_set_uint,
          G_MAXUINT), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_UINT, g_value_set_uint,
          (G_MAXUINT / 2)), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_UINT, g_value_set_uint,
          0), BUFFER_LENGTH, 2);
}

GST_START_TEST (marshal_uint)
{
  test_uint_GValue (KURENTO_MARSHALL_BER);
  test_uint_GValue (KURENTO_MARSHALL_XER);
  test_uint_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_uint64_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_UINT64,
          g_value_set_uint64, G_MAXUINT64), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_UINT64,
          g_value_set_uint64, 456789), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_UINT64,
          g_value_set_uint64, 0), BUFFER_LENGTH, 2);
}

GST_START_TEST (marshal_uint64)
{
  test_uint64_GValue (KURENTO_MARSHALL_BER);
  test_uint64_GValue (KURENTO_MARSHALL_XER);
  test_uint64_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_boolean_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_BOOLEAN,
          g_value_set_boolean, TRUE), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_BOOLEAN,
          g_value_set_boolean, FALSE), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_BOOLEAN,
          g_value_set_boolean, FALSE), BUFFER_LENGTH, 2);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_BOOLEAN,
          g_value_set_boolean, TRUE), BUFFER_LENGTH, 3);
}

GST_START_TEST (marshal_boolean)
{
  test_boolean_GValue (KURENTO_MARSHALL_BER);
  test_boolean_GValue (KURENTO_MARSHALL_XER);
  test_boolean_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_float_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_FLOAT, g_value_set_float,
          G_MAXFLOAT), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_FLOAT, g_value_set_float,
          (G_MAXFLOAT / 2.0)), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_FLOAT, g_value_set_float,
          0.0), BUFFER_LENGTH, 2);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_FLOAT, g_value_set_float,
          (G_MINFLOAT / 2.0)), BUFFER_LENGTH, 3);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_FLOAT, g_value_set_float,
          G_MINFLOAT), BUFFER_LENGTH, 4);
}

GST_START_TEST (marshal_float)
{
  test_float_GValue (KURENTO_MARSHALL_BER);
  /* Set bigger buffer to enable this test */
//   test_float_GValue (KURENTO_MARSHALL_XER);
  test_float_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_string_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_STRING,
          g_value_set_static_string, NULL), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_STRING,
          g_value_set_static_string, ""), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_STRING,
          g_value_set_static_string,
          "tes1:x-raw, format (fraction) = 1/5, val (range) = [1, 4]"),
      BUFFER_LENGTH, 2);
}

GST_START_TEST (marshal_string)
{
  test_string_GValue (KURENTO_MARSHALL_BER);
  test_string_GValue (KURENTO_MARSHALL_XER);
  test_string_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_fraction_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_FRACTION,
          gst_value_set_fraction, G_MAXINT, G_MAXINT), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_FRACTION,
          gst_value_set_fraction, G_MAXINT, 1), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_FRACTION,
          gst_value_set_fraction, G_MAXINT, -G_MAXINT), BUFFER_LENGTH, 2);
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_FRACTION,
          gst_value_set_fraction, -G_MAXINT, -G_MAXINT), BUFFER_LENGTH, 3);
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_FRACTION,
          gst_value_set_fraction, -G_MAXINT, 6), BUFFER_LENGTH, 4);
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_FRACTION,
          gst_value_set_fraction, G_MAXINT, 65437), BUFFER_LENGTH, 5);
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_FRACTION,
          gst_value_set_fraction, 63456, -G_MAXINT), BUFFER_LENGTH, 6);
}

GST_START_TEST (marshal_fraction)
{
  test_fraction_GValue (KURENTO_MARSHALL_BER);
  test_fraction_GValue (KURENTO_MARSHALL_XER);
  test_fraction_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_int_range_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_INT_RANGE,
          gst_value_set_int_range, G_MININT, G_MAXINT), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_INT_RANGE,
          gst_value_set_int_range, 0, G_MAXINT), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_INT_RANGE,
          gst_value_set_int_range, G_MININT, 0), BUFFER_LENGTH, 2);
}

GST_START_TEST (marshal_int_range)
{
  test_int_range_GValue (KURENTO_MARSHALL_BER);
  test_int_range_GValue (KURENTO_MARSHALL_XER);
  test_int_range_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_double_range_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_DOUBLE_RANGE,
          gst_value_set_double_range, G_MINDOUBLE, G_MAXDOUBLE),
      BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_DOUBLE_RANGE,
          gst_value_set_double_range, G_MINDOUBLE, 0.01), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_RANGE_GVALUE (GST_TYPE_DOUBLE_RANGE,
          gst_value_set_double_range, 0.0, G_MAXDOUBLE), BUFFER_LENGTH, 2);
}

GST_START_TEST (marshal_double_range)
{
  test_double_range_GValue (KURENTO_MARSHALL_BER);
  test_double_range_GValue (KURENTO_MARSHALL_XER);
  test_double_range_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_fraction_range_GValue (KurentoMarshalRules rules)
{

  GValue *val1, *val2, *min, *max;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = val2 = NULL;
  min = g_slice_new0 (GValue);
  max = g_slice_new0 (GValue);

  g_value_init (min, GST_TYPE_FRACTION);
  g_value_init (max, GST_TYPE_FRACTION);

  gst_value_set_fraction (min, 6, -G_MAXINT);
  gst_value_set_fraction (max, G_MAXINT, 6);

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_FRACTION_RANGE);
  gst_value_set_fraction_range (val1, min, max);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a fraction range value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a fraction range value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

  gst_value_set_fraction (min, -4, 3);
  gst_value_set_fraction (max, 6, 7);
  gst_value_set_fraction_range (val1, min, max);

  g_value_unset (val2);
  g_slice_free (GValue, val2);
  val2 = NULL;

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a fraction range value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a fraction range value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (min != NULL)
    g_slice_free (GValue, min);

  if (max != NULL)
    g_slice_free (GValue, max);

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

GST_START_TEST (marshal_fraction_range)
{
  test_fraction_range_GValue (KURENTO_MARSHALL_BER);
  test_fraction_range_GValue (KURENTO_MARSHALL_XER);
  test_fraction_range_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_empty_list_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_LIST);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a list value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a list value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_int_list_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_LIST);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, G_TYPE_INT);

  g_value_set_int (val, 15);
  gst_value_list_append_value (val1, val);

  g_value_set_int (val, G_MAXINT);
  gst_value_list_append_value (val1, val);

  g_value_set_int (val, G_MININT);
  gst_value_list_append_value (val1, val);

  g_value_set_int (val, 0);
  gst_value_list_append_value (val1, val);

  g_value_set_int (val, -3452);
  gst_value_list_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a list value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a list value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_boolean_list_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_LIST);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, G_TYPE_BOOLEAN);

  g_value_set_boolean (val, TRUE);
  gst_value_list_append_value (val1, val);

  g_value_set_boolean (val, FALSE);
  gst_value_list_append_value (val1, val);

  g_value_set_boolean (val, TRUE);
  gst_value_list_append_value (val1, val);

  g_value_set_boolean (val, FALSE);
  gst_value_list_append_value (val1, val);

  g_value_set_boolean (val, TRUE);
  gst_value_list_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a list value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a list value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_float_list_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_LIST);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, G_TYPE_FLOAT);

  g_value_set_float (val, -0.0);
  gst_value_list_append_value (val1, val);

  g_value_set_float (val, -234.54);
  gst_value_list_append_value (val1, val);

  g_value_set_float (val, -32454.45);
  gst_value_list_append_value (val1, val);

  g_value_set_float (val, -11923.5432);
  gst_value_list_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a list value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a list value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_string_list_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_LIST);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, G_TYPE_STRING);

  g_value_set_static_string (val, NULL);
  gst_value_list_append_value (val1, val);

  g_value_set_static_string (val, "");
  gst_value_list_append_value (val1, val);

  g_value_set_static_string (val, "hello");
  gst_value_list_append_value (val1, val);

  g_value_set_static_string (val, "fdslk'0(9&456:=235476");
  gst_value_list_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a list value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a list value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_fraction_list_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_LIST);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, GST_TYPE_FRACTION);

  gst_value_set_fraction (val, G_MAXINT, 6);
  gst_value_list_append_value (val1, val);

  gst_value_set_fraction (val, 342, -G_MAXINT);
  gst_value_list_append_value (val1, val);

  gst_value_set_fraction (val, G_MAXINT, 423354);
  gst_value_list_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a list value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a list value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_int_range_list_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_LIST);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, GST_TYPE_INT_RANGE);

  gst_value_set_int_range (val, G_MININT, G_MAXINT);
  gst_value_list_append_value (val1, val);

  gst_value_set_int_range (val, G_MININT, 0);
  gst_value_list_append_value (val1, val);

  gst_value_set_int_range (val, 0, G_MAXINT);
  gst_value_list_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a list value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a list value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_double_range_list_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_LIST);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, GST_TYPE_DOUBLE_RANGE);

  gst_value_set_double_range (val, G_MINDOUBLE, G_MAXDOUBLE);
  gst_value_list_append_value (val1, val);

  gst_value_set_double_range (val, G_MINDOUBLE, 0.01);
  gst_value_list_append_value (val1, val);

  gst_value_set_double_range (val, 0, G_MAXDOUBLE);
  gst_value_list_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a list value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a list value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_fraction_range_list_GValue (KurentoMarshalRules rules)
{
  GValue *val1, *val2, *min, *max;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = val2 = NULL;
  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_LIST);

  min = g_slice_new0 (GValue);
  max = g_slice_new0 (GValue);

  g_value_init (min, GST_TYPE_FRACTION);
  g_value_init (max, GST_TYPE_FRACTION);

  gst_value_set_fraction (min, 6, -G_MAXINT);
  gst_value_set_fraction (max, G_MAXINT, 6);

  val = g_slice_new0 (GValue);
  g_value_init (val, GST_TYPE_FRACTION_RANGE);
  gst_value_set_fraction_range (val, min, max);

  gst_value_list_append_value (val1, val);

  gst_value_set_fraction (min, G_MAXINT, -G_MAXINT);
  gst_value_set_fraction (max, 24, 3);
  gst_value_set_fraction_range (val, min, max);
  gst_value_list_append_value (val1, val);

  gst_value_set_fraction (min, 1, 5);
  gst_value_set_fraction (max, 9, 3);
  gst_value_set_fraction_range (val, min, max);
  gst_value_list_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a list value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a list value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (min != NULL)
    g_slice_free (GValue, min);

  if (max != NULL)
    g_slice_free (GValue, max);

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_list_of_lists_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *list, *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_LIST);

  list = g_slice_new0 (GValue);
  g_value_init (list, GST_TYPE_LIST);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, G_TYPE_STRING);

  g_value_set_static_string (val, "item1");
  gst_value_list_append_value (list, val);

  g_value_set_static_string (val, "item2");
  gst_value_list_append_value (list, val);

  g_value_set_static_string (val, "item3");
  gst_value_list_append_value (list, val);

  gst_value_list_append_value (val1, list);

  g_value_unset (list);
  g_value_init (list, GST_TYPE_LIST);

  g_value_set_static_string (val, NULL);
  gst_value_list_append_value (list, val);

  g_value_set_static_string (val, "");
  gst_value_list_append_value (list, val);

  g_value_set_static_string (val, "item4");
  gst_value_list_append_value (list, val);

  gst_value_list_append_value (val1, list);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a list value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a list value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (list != NULL) {
    g_value_unset (list);
    g_slice_free (GValue, list);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

GST_START_TEST (marshal_list)
{
  test_empty_list_GValue (KURENTO_MARSHALL_BER);
  test_empty_list_GValue (KURENTO_MARSHALL_PER);

  test_int_list_GValue (KURENTO_MARSHALL_BER);
  test_int_list_GValue (KURENTO_MARSHALL_PER);

  test_boolean_list_GValue (KURENTO_MARSHALL_BER);
  test_boolean_list_GValue (KURENTO_MARSHALL_PER);

  test_float_list_GValue (KURENTO_MARSHALL_BER);
  test_float_list_GValue (KURENTO_MARSHALL_PER);

  test_string_list_GValue (KURENTO_MARSHALL_BER);
  test_string_list_GValue (KURENTO_MARSHALL_PER);

  test_fraction_list_GValue (KURENTO_MARSHALL_BER);
  test_fraction_list_GValue (KURENTO_MARSHALL_PER);

  test_int_range_list_GValue (KURENTO_MARSHALL_BER);
  test_int_range_list_GValue (KURENTO_MARSHALL_PER);

  test_double_range_list_GValue (KURENTO_MARSHALL_BER);
  test_double_range_list_GValue (KURENTO_MARSHALL_PER);

  test_fraction_range_list_GValue (KURENTO_MARSHALL_BER);
  test_fraction_range_list_GValue (KURENTO_MARSHALL_PER);

  test_list_of_lists_GValue (KURENTO_MARSHALL_BER);
  test_list_of_lists_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_empty_array_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_ARRAY);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a array value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a array value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_int_array_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_ARRAY);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, G_TYPE_INT);

  g_value_set_int (val, 15);
  gst_value_array_append_value (val1, val);

  g_value_set_int (val, G_MAXINT);
  gst_value_array_append_value (val1, val);

  g_value_set_int (val, G_MININT);
  gst_value_array_append_value (val1, val);

  g_value_set_int (val, 0);
  gst_value_array_append_value (val1, val);

  g_value_set_int (val, -3452);
  gst_value_array_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a array value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a array value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_boolean_array_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_ARRAY);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, G_TYPE_BOOLEAN);

  g_value_set_boolean (val, TRUE);
  gst_value_array_append_value (val1, val);

  g_value_set_boolean (val, FALSE);
  gst_value_array_append_value (val1, val);

  g_value_set_boolean (val, TRUE);
  gst_value_array_append_value (val1, val);

  g_value_set_boolean (val, FALSE);
  gst_value_array_append_value (val1, val);

  g_value_set_boolean (val, TRUE);
  gst_value_array_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a array value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a array value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_float_array_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_ARRAY);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, G_TYPE_FLOAT);

  g_value_set_float (val, -0.0);
  gst_value_array_append_value (val1, val);

  g_value_set_float (val, -234.54);
  gst_value_array_append_value (val1, val);

  g_value_set_float (val, -32454.45);
  gst_value_array_append_value (val1, val);

  g_value_set_float (val, -11923.5432);
  gst_value_array_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a array value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a array value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_string_array_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_ARRAY);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, G_TYPE_STRING);

  g_value_set_static_string (val, NULL);
  gst_value_array_append_value (val1, val);

  g_value_set_static_string (val, "");
  gst_value_array_append_value (val1, val);

  g_value_set_static_string (val, "hello");
  gst_value_array_append_value (val1, val);

  g_value_set_static_string (val, "fdslk'0(9&456:=235476");
  gst_value_array_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a array value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a array value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_fraction_array_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_ARRAY);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, GST_TYPE_FRACTION);

  gst_value_set_fraction (val, G_MAXINT, 6);
  gst_value_array_append_value (val1, val);

  gst_value_set_fraction (val, 342, -G_MAXINT);
  gst_value_array_append_value (val1, val);

  gst_value_set_fraction (val, G_MAXINT, 423354);
  gst_value_array_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a array value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a array value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_int_range_array_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_ARRAY);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, GST_TYPE_INT_RANGE);

  gst_value_set_int_range (val, G_MININT, G_MAXINT);
  gst_value_array_append_value (val1, val);

  gst_value_set_int_range (val, G_MININT, 0);
  gst_value_array_append_value (val1, val);

  gst_value_set_int_range (val, 0, G_MAXINT);
  gst_value_array_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a array value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a array value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_double_range_array_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_ARRAY);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, GST_TYPE_DOUBLE_RANGE);

  gst_value_set_double_range (val, G_MINDOUBLE, G_MAXDOUBLE);
  gst_value_array_append_value (val1, val);

  gst_value_set_double_range (val, G_MINDOUBLE, 0.01);
  gst_value_array_append_value (val1, val);

  gst_value_set_double_range (val, 0, G_MAXDOUBLE);
  gst_value_array_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a array value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a array value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_fraction_range_array_GValue (KurentoMarshalRules rules)
{
  GValue *val1, *val2, *min, *max;
  GValue *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = val2 = NULL;
  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_ARRAY);

  min = g_slice_new0 (GValue);
  max = g_slice_new0 (GValue);

  g_value_init (min, GST_TYPE_FRACTION);
  g_value_init (max, GST_TYPE_FRACTION);

  gst_value_set_fraction (min, 6, -G_MAXINT);
  gst_value_set_fraction (max, G_MAXINT, 6);

  val = g_slice_new0 (GValue);
  g_value_init (val, GST_TYPE_FRACTION_RANGE);
  gst_value_set_fraction_range (val, min, max);

  gst_value_array_append_value (val1, val);

  gst_value_set_fraction (min, G_MAXINT, -G_MAXINT);
  gst_value_set_fraction (max, 24, 3);
  gst_value_set_fraction_range (val, min, max);
  gst_value_array_append_value (val1, val);

  gst_value_set_fraction (min, 1, 5);
  gst_value_set_fraction (max, 9, 3);
  gst_value_set_fraction_range (val, min, max);
  gst_value_array_append_value (val1, val);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a array value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a array value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (min != NULL)
    g_slice_free (GValue, min);

  if (max != NULL)
    g_slice_free (GValue, max);

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

static void
test_array_of_arrays_GValue (KurentoMarshalRules rules)
{
  GValue *val1 = NULL, *val2 = NULL;
  GValue *array, *val;
  GError *err = NULL;
  gchar buf[BUFFER_LENGTH];
  gsize size;

  val1 = g_slice_new0 (GValue);
  g_value_init (val1, GST_TYPE_ARRAY);

  array = g_slice_new0 (GValue);
  g_value_init (array, GST_TYPE_ARRAY);

  /* Add an integers value */
  val = g_slice_new0 (GValue);
  g_value_init (val, G_TYPE_STRING);

  g_value_set_static_string (val, "item1");
  gst_value_array_append_value (array, val);

  g_value_set_static_string (val, "item2");
  gst_value_array_append_value (array, val);

  g_value_set_static_string (val, "item3");
  gst_value_array_append_value (array, val);

  gst_value_array_append_value (val1, array);

  g_value_unset (array);
  g_value_init (array, GST_TYPE_ARRAY);

  g_value_set_static_string (val, NULL);
  gst_value_array_append_value (array, val);

  g_value_set_static_string (val, "");
  gst_value_array_append_value (array, val);

  g_value_set_static_string (val, "item4");
  gst_value_array_append_value (array, val);

  gst_value_array_append_value (val1, array);

  /* Encode GVal 1 */
  size = enc_GValue (rules, val1, buf, BUFFER_LENGTH, &err);
  if (err != NULL) {
    GST_ERROR ("%s", err->message);
    fail ("Failed encoding GValue holding a array value");
    goto end;
  }

  /* Encode GVal 2 */
  if (!dec_GValue (rules, buf, size, &val2, &err)) {
    GST_ERROR ("%s", err->message);
    fail ("Failed decoding GValue holding a array value");
    goto end;
  }

  if (!gvalue_are_equal (val1, val2)) {
    GST_ERROR ("Value encoded and value decoded are not the same");
    fail ("Marshal did not succeed");
  }

end:
  if (err != NULL)
    g_error_free (err);

  if (val != NULL) {
    g_value_unset (val);
    g_slice_free (GValue, val);
  }

  if (array != NULL) {
    g_value_unset (array);
    g_slice_free (GValue, array);
  }

  if (val1 != NULL) {
    g_value_unset (val1);
    g_slice_free (GValue, val1);
  }

  if (val2 != NULL) {
    g_value_unset (val2);
    g_slice_free (GValue, val2);
  }
}

GST_START_TEST (marshal_array)
{
  test_empty_array_GValue (KURENTO_MARSHALL_BER);
  test_empty_array_GValue (KURENTO_MARSHALL_PER);

  test_int_array_GValue (KURENTO_MARSHALL_BER);
  test_int_array_GValue (KURENTO_MARSHALL_PER);

  test_boolean_array_GValue (KURENTO_MARSHALL_BER);
  test_boolean_array_GValue (KURENTO_MARSHALL_PER);

  test_float_array_GValue (KURENTO_MARSHALL_BER);
  test_float_array_GValue (KURENTO_MARSHALL_PER);

  test_string_array_GValue (KURENTO_MARSHALL_BER);
  test_string_array_GValue (KURENTO_MARSHALL_PER);

  test_fraction_array_GValue (KURENTO_MARSHALL_BER);
  test_fraction_array_GValue (KURENTO_MARSHALL_PER);

  test_int_range_array_GValue (KURENTO_MARSHALL_BER);
  test_int_range_array_GValue (KURENTO_MARSHALL_PER);

  test_double_range_array_GValue (KURENTO_MARSHALL_BER);
  test_double_range_array_GValue (KURENTO_MARSHALL_PER);

  test_fraction_range_array_GValue (KURENTO_MARSHALL_BER);
  test_fraction_range_array_GValue (KURENTO_MARSHALL_PER);

  test_array_of_arrays_GValue (KURENTO_MARSHALL_BER);
  test_array_of_arrays_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_int64_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_INT64,
          g_value_set_int64, G_MAXINT64), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_INT64,
          g_value_set_int64, G_MININT64), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_INT64,
          g_value_set_int64, 0), BUFFER_LENGTH, 2);
}

GST_START_TEST (marshal_int64)
{
  test_int64_GValue (KURENTO_MARSHALL_BER);
  test_int64_GValue (KURENTO_MARSHALL_XER);
  test_int64_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_stream_flags_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_STREAM_FLAGS,
          g_value_set_flags, GST_STREAM_FLAG_NONE), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_STREAM_FLAGS,
          g_value_set_flags, GST_STREAM_FLAG_SPARSE), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_STREAM_FLAGS,
          g_value_set_flags, GST_STREAM_FLAG_SELECT), BUFFER_LENGTH, 2);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_STREAM_FLAGS,
          g_value_set_flags, GST_STREAM_FLAG_UNSELECT), BUFFER_LENGTH, 3);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_STREAM_FLAGS,
          g_value_set_flags, (GST_STREAM_FLAG_SPARSE | GST_STREAM_FLAG_SELECT)),
      BUFFER_LENGTH, 4);
}

GST_START_TEST (marshal_stream_flags)
{
  test_stream_flags_GValue (KURENTO_MARSHALL_BER);
  test_stream_flags_GValue (KURENTO_MARSHALL_XER);
  test_stream_flags_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_format_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_FORMAT,
          g_value_set_enum, GST_FORMAT_UNDEFINED), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_FORMAT,
          g_value_set_enum, GST_FORMAT_DEFAULT), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_FORMAT,
          g_value_set_enum, GST_FORMAT_BYTES), BUFFER_LENGTH, 2);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_FORMAT,
          g_value_set_enum, GST_FORMAT_TIME), BUFFER_LENGTH, 3);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_FORMAT,
          g_value_set_enum, GST_FORMAT_BUFFERS), BUFFER_LENGTH, 4);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_FORMAT,
          g_value_set_enum, GST_FORMAT_PERCENT), BUFFER_LENGTH, 5);
}

GST_START_TEST (marshal_format)
{
  test_format_GValue (KURENTO_MARSHALL_BER);
  test_format_GValue (KURENTO_MARSHALL_XER);
  test_format_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_qos_type_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_QOS_TYPE,
          g_value_set_enum, GST_QOS_TYPE_OVERFLOW), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_QOS_TYPE,
          g_value_set_enum, GST_QOS_TYPE_UNDERFLOW), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_QOS_TYPE,
          g_value_set_enum, GST_QOS_TYPE_THROTTLE), BUFFER_LENGTH, 2);
}

GST_START_TEST (marshal_qos_type)
{
  test_qos_type_GValue (KURENTO_MARSHALL_BER);
  test_qos_type_GValue (KURENTO_MARSHALL_XER);
  test_qos_type_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_double_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_DOUBLE,
          g_value_set_double, G_MAXDOUBLE), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_DOUBLE,
          g_value_set_double, G_MINDOUBLE), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_DOUBLE,
          g_value_set_double, 0.0), BUFFER_LENGTH, 2);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_DOUBLE,
          g_value_set_double, -34665.6454), BUFFER_LENGTH, 3);
}

GST_START_TEST (marshal_double)
{
  test_double_GValue (KURENTO_MARSHALL_BER);
  test_double_GValue (KURENTO_MARSHALL_XER);
  test_double_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_date_GValue (KurentoMarshalRules rules)
{
  GDate *date;

  date = g_date_new_julian (2345);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_DATE,
          g_value_set_boxed, date), BUFFER_LENGTH, 0);

  g_date_set_dmy (date, 23, 1, 1982);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_DATE,
          g_value_set_boxed, date), BUFFER_LENGTH, 1);

  g_date_set_julian (date, 1218764);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_DATE,
          g_value_set_boxed, date), BUFFER_LENGTH, 2);

  g_date_free (date);
}

GST_START_TEST (marshal_date)
{
  test_date_GValue (KURENTO_MARSHALL_BER);
  test_date_GValue (KURENTO_MARSHALL_XER);
  test_date_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_seek_flags_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEEK_FLAGS,
          g_value_set_flags, GST_SEEK_FLAG_NONE), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEEK_FLAGS,
          g_value_set_flags, GST_SEEK_FLAG_FLUSH), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEEK_FLAGS,
          g_value_set_flags, GST_SEEK_FLAG_ACCURATE), BUFFER_LENGTH, 2);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEEK_FLAGS,
          g_value_set_flags, GST_SEEK_FLAG_KEY_UNIT), BUFFER_LENGTH, 3);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEEK_FLAGS,
          g_value_set_flags, GST_SEEK_FLAG_SEGMENT), BUFFER_LENGTH, 4);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEEK_FLAGS,
          g_value_set_flags, GST_SEEK_FLAG_SKIP), BUFFER_LENGTH, 5);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEEK_FLAGS,
          g_value_set_flags, GST_SEEK_FLAG_SNAP_BEFORE), BUFFER_LENGTH, 6);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEEK_FLAGS,
          g_value_set_flags, GST_SEEK_FLAG_SNAP_AFTER), BUFFER_LENGTH, 7);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEEK_FLAGS,
          g_value_set_flags, GST_SEEK_FLAG_SNAP_NEAREST), BUFFER_LENGTH, 8);
}

GST_START_TEST (marshal_seek_flags)
{
  test_seek_flags_GValue (KURENTO_MARSHALL_BER);
  test_seek_flags_GValue (KURENTO_MARSHALL_XER);
  test_seek_flags_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_seek_type_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEEK_TYPE,
          g_value_set_enum, GST_SEEK_TYPE_NONE), BUFFER_LENGTH, 0);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEEK_TYPE,
          g_value_set_enum, GST_SEEK_TYPE_SET), BUFFER_LENGTH, 1);
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEEK_TYPE,
          g_value_set_enum, GST_SEEK_TYPE_END), BUFFER_LENGTH, 2);
}

GST_START_TEST (marshal_seek_type)
{
  test_seek_type_GValue (KURENTO_MARSHALL_BER);
  test_seek_type_GValue (KURENTO_MARSHALL_XER);
  test_seek_type_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_tag_list_GValue (KurentoMarshalRules rules)
{
  GstTagList *taglist;
  GstDateTime *datetime;
  GDate *date;

  datetime = gst_date_time_new_now_utc ();
  date = g_date_new_dmy (23, 1, 1982);

  taglist = gst_tag_list_new_empty ();
  gst_tag_list_add (taglist, GST_TAG_MERGE_APPEND,
      GST_TAG_TITLE, "test1",
      GST_TAG_TITLE, "test2",
      GST_TAG_TITLE_SORTNAME, "test-title",
      GST_TAG_ALBUM, "test-album",
      GST_TAG_DATE, date,
      GST_TAG_DATE_TIME, datetime,
      GST_TAG_SHOW_SEASON_NUMBER, 2,
      GST_TAG_SHOW_EPISODE_NUMBER, G_MAXUINT,
      GST_TAG_IMAGE_ORIENTATION, "flip-rotate-270",
      GST_TAG_USER_RATING, 10,
      GST_TAG_GEO_LOCATION_HORIZONTAL_ERROR, 3.0, NULL);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_TAG_LIST,
          g_value_set_boxed, taglist), BUFFER_LENGTH, 0);

  gst_tag_list_unref (taglist);
  gst_date_time_unref (datetime);
  g_date_free (date);
}

GST_START_TEST (marshal_tag_list)
{
  test_tag_list_GValue (KURENTO_MARSHALL_BER);
//   test_tag_list_GValue (KURENTO_MARSHALL_XER);
  test_tag_list_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_date_time_GValue (KurentoMarshalRules rules)
{
  GstDateTime *datetime;

  datetime = gst_date_time_new_now_utc ();

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_DATE_TIME,
          g_value_set_boxed, datetime), BUFFER_LENGTH, 0);

  gst_date_time_unref (datetime);
}

GST_START_TEST (marshal_date_time)
{
  test_date_time_GValue (KURENTO_MARSHALL_BER);
  test_date_time_GValue (KURENTO_MARSHALL_XER);
  test_date_time_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_null_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_CAPS,
          g_value_set_boxed, NULL), BUFFER_LENGTH, 0);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (G_TYPE_DATE,
          g_value_set_boxed, NULL), BUFFER_LENGTH, 1);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_TAG_LIST,
          g_value_set_boxed, NULL), BUFFER_LENGTH, 2);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SEGMENT,
          g_value_set_boxed, NULL), BUFFER_LENGTH, 3);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_TOC,
          g_value_set_boxed, NULL), BUFFER_LENGTH, 4);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_DATE_TIME,
          g_value_set_boxed, NULL), BUFFER_LENGTH, 5);
}

GST_START_TEST (marshal_null_gvalues)
{
  test_null_GValue (KURENTO_MARSHALL_BER);
  test_null_GValue (KURENTO_MARSHALL_XER);
  test_null_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_scheduling_flags_GValue (KurentoMarshalRules rules)
{
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SCHEDULING_FLAGS,
          g_value_set_flags, GST_SCHEDULING_FLAG_SEEKABLE), BUFFER_LENGTH, 0);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SCHEDULING_FLAGS,
          g_value_set_flags, GST_SCHEDULING_FLAG_SEQUENTIAL), BUFFER_LENGTH, 1);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SCHEDULING_FLAGS,
          g_value_set_flags, GST_SCHEDULING_FLAG_BANDWIDTH_LIMITED),
      BUFFER_LENGTH, 2);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SCHEDULING_FLAGS,
          g_value_set_flags, 0), BUFFER_LENGTH, 3);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_SCHEDULING_FLAGS,
          g_value_set_flags,
          (GST_SCHEDULING_FLAG_SEQUENTIAL | GST_SCHEDULING_FLAG_SEEKABLE)),
      BUFFER_LENGTH, 4);
}

GST_START_TEST (marshal_scheduling_flags)
{
  test_scheduling_flags_GValue (KURENTO_MARSHALL_BER);
  test_scheduling_flags_GValue (KURENTO_MARSHALL_XER);
  test_scheduling_flags_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST static void
test_context_GValue (KurentoMarshalRules rules)
{
  GstContext *context;
  GstStructure *structure;

  context = gst_context_new ("test", TRUE);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_CONTEXT,
          g_value_set_boxed, context), BUFFER_LENGTH, 0);

  structure = gst_context_writable_structure (context);
  gst_structure_set (structure, "foobar", G_TYPE_INT, 1, NULL);

  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_CONTEXT,
          g_value_set_boxed, context), BUFFER_LENGTH, 1);

  gst_context_unref (context);
}

GST_START_TEST (marshal_context)
{
  test_context_GValue (KURENTO_MARSHALL_BER);
  test_context_GValue (KURENTO_MARSHALL_XER);
  test_context_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST
#define BUFFER_TEST_CASE(buff)                                       \
({                                                                   \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 0);               \
                                                                     \
  GST_BUFFER_FLAG_SET (buff, GST_BUFFER_FLAG_LIVE);                  \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 1);               \
                                                                     \
  GST_BUFFER_FLAG_SET (buff, GST_BUFFER_FLAG_DECODE_ONLY);           \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 2);               \
                                                                     \
  GST_BUFFER_FLAG_SET (buff, GST_BUFFER_FLAG_DISCONT);               \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 3);               \
                                                                     \
  GST_BUFFER_FLAG_SET (buff, GST_BUFFER_FLAG_RESYNC);                \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 4);               \
                                                                     \
  GST_BUFFER_FLAG_SET (buff, GST_BUFFER_FLAG_CORRUPTED);             \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 5);               \
                                                                     \
  GST_BUFFER_FLAG_SET (buff, GST_BUFFER_FLAG_MARKER);                \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 6);               \
                                                                     \
  GST_BUFFER_FLAG_SET (buff, GST_BUFFER_FLAG_HEADER);                \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 7);               \
                                                                     \
  GST_BUFFER_FLAG_SET (buff, GST_BUFFER_FLAG_GAP);                   \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 8);               \
                                                                     \
  GST_BUFFER_FLAG_SET (buff, GST_BUFFER_FLAG_DROPPABLE);             \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 9);               \
                                                                     \
  GST_BUFFER_FLAG_SET (buff, GST_BUFFER_FLAG_DELTA_UNIT);            \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 10);              \
                                                                     \
  GST_BUFFER_FLAG_SET (buff, GST_BUFFER_FLAG_TAG_MEMORY);            \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 11);              \
                                                                     \
  GST_BUFFER_FLAG_SET (buff, (GST_BUFFER_FLAG_TAG_MEMORY |           \
          GST_BUFFER_FLAG_DELTA_UNIT | GST_BUFFER_FLAG_RESYNC));     \
  TEST_BASE_CASE (rules, CREATE_SINGLE_GVALUE (GST_TYPE_BUFFER,      \
          g_value_set_boxed, buff), BUFFER_LENGTH, 12);              \
})
    static void
test_empty_buffer_GValue (KurentoMarshalRules rules)
{
  GstBuffer *buff;

  buff = gst_buffer_new ();

  BUFFER_TEST_CASE (buff);

  gst_buffer_unref (buff);
}

static void
test_buffer_GValue (KurentoMarshalRules rules)
{
  GstBuffer *buff;

  buff = gst_buffer_new_wrapped (g_strdup (TEST_BUFFER), strlen (TEST_BUFFER));

  BUFFER_TEST_CASE (buff);

  gst_buffer_unref (buff);
}

GST_START_TEST (marshal_buffer_gvalues)
{
  test_empty_buffer_GValue (KURENTO_MARSHALL_BER);
//   test_context_GValue (KURENTO_MARSHALL_XER);
  test_empty_buffer_GValue (KURENTO_MARSHALL_PER);

  test_buffer_GValue (KURENTO_MARSHALL_BER);
//   test_buffer_GValue (KURENTO_MARSHALL_XER);
  test_buffer_GValue (KURENTO_MARSHALL_PER);
}

GST_END_TEST
/*********************************/
/* gvalue_marshalling test suite */
/*********************************/
static Suite *
gvalue_marshalling_suite (void)
{
  Suite *s = suite_create ("gvalue");
  TCase *tc_chain = tcase_create ("marshalling");

  suite_add_tcase (s, tc_chain);
  tcase_add_test (tc_chain, marshal_int);
  tcase_add_test (tc_chain, marshal_uint);
  tcase_add_test (tc_chain, marshal_boolean);
  tcase_add_test (tc_chain, marshal_float);
  tcase_add_test (tc_chain, marshal_string);
  tcase_add_test (tc_chain, marshal_fraction);
  tcase_add_test (tc_chain, marshal_int_range);
  tcase_add_test (tc_chain, marshal_double_range);
  tcase_add_test (tc_chain, marshal_fraction_range);
  tcase_add_test (tc_chain, marshal_list);
  tcase_add_test (tc_chain, marshal_array);
  tcase_add_test (tc_chain, marshal_uint64);
  tcase_add_test (tc_chain, marshal_int64);
  tcase_add_test (tc_chain, marshal_stream_flags);
  tcase_add_test (tc_chain, marshal_format);
  tcase_add_test (tc_chain, marshal_qos_type);
  tcase_add_test (tc_chain, marshal_date);
  tcase_add_test (tc_chain, marshal_double);
  tcase_add_test (tc_chain, marshal_seek_flags);
  tcase_add_test (tc_chain, marshal_seek_type);
  tcase_add_test (tc_chain, marshal_tag_list);
  tcase_add_test (tc_chain, marshal_date_time);
  tcase_add_test (tc_chain, marshal_scheduling_flags);
  tcase_add_test (tc_chain, marshal_context);
  tcase_add_test (tc_chain, marshal_null_gvalues);
  tcase_add_test (tc_chain, marshal_buffer_gvalues);

  return s;
}

GST_CHECK_MAIN (gvalue_marshalling);
